//-----------------------------------------------------------------------------
// Sprache    : C
// Version    : 1.0
// Autor      : Christian Mller - www.myAVR.de
// Prozessor  : ATmega2560
// Takt       : 16 MHz
// Programmer : mySmartUSB MK3
//-----------------------------------------------------------------------------
// Titel      : LCD-Basics
//				Sample W03 - myAVR Beispiel fr MK3 Board
//				(mehr und ausfhlicher im SiSy-Projekt im Download unter www.myAVR.de) 
// title      : 7-Segment-display
//				sample W03 - myAVR sample for the MK3 board
//				(more details in the SiSy-project at download on www.myAVR.de)
//-----------------------------------------------------------------------------
// Funktion   : Zeigt anhand verschiedener Beispiele auf dem Grafik-LCD
// function   : Shows different samples for a output on the grafic-LCD 
// Schaltung  : Quick-Connect "LCD on Port C+A" und "LED on Port-L" auf dem MK3 Board
// circuit    : Quick-Connect "LCD on Port C+A" and "LED on Port-L" at the MK3 Board
//-----------------------------------------------------------------------------
// LCD-Informationen 
// 
//	PortC = LCD-Data
//	PortA = LCD-Control
//			A7 = PS (std=High=parallel Mode)
//			A6 = C86 (std=High=6800 Mode)
//			A5 = unbenutzt
//			A4 = /RD = E (Schreibflanke H->L)
//			A3 = /WR = R/W (Low=Schreiben)
//			A2 = A0 (Low=ControlData; High=DisplayData)
//			A1 = /RES (Low=activ; std=High)
//			A0 = CS1 (ChipSelect=muss Low sein; std=High)
//
// LCD-information 
// 
//	PortC = LCD-data
//	PortA = LCD-control
//			A7 = PS (std=high=parallel mode)
//			A6 = C86 (std=high=6800 mode)
//			A5 = unused
//			A4 = /RD = E (writing edge H->L)
//			A3 = /WR = R/W (low=write)
//			A2 = A0 (low=controldata; high=displaydata)
//			A1 = /RES (low=activ; std=high)
//			A0 = CS1 (chipselect=must be low; std=high)
//--------------------------------------------------------------------------
#define 	F_CPU 	16000000
#include <avr\io.h>
#include	<avr/io.h>
void uartPutString(char *buffer);
//--------------------------------------------------------------------------
//de:  defines fr mehr bersicht welcher Port fr was zustndig ist
//eng: defines for a better abridgement which port what do 
#define PortLcdData PORTC
#define PortLcdDataPIN PINC
#define PortLcdDataDDR DDRC
#define PortLcdCtrl PORTA
#define PortLcdCtrlDDR DDRA
#define PortLed PORTL
#define PortLedDDR DDRL
//de:  initalisieren der Ports
//eng: initialise the ports 
void init()
{
	//de:  Port-L als Ausgang konfiguriren (fr die LEDs)
	//eng: config port-L as output (for the LEDs)
	PortLedDDR=0xff;

	//de:  Port-C als Eingang ohne Pullup konfigurieren (fr LCD-Data)
	//de:  Port-C mu je nach Aktion die ausgefhrt wird (schreiben oder lesen) als Ausgang oder Eingang konfiguriert werden
	//eng: config port-C as input whitout pullup (for LCD-Data)
	//eng: port-C must be config as output while reading and output while writing
	PortLcdDataDDR=0x00;
	PortLcdData=0;

	//de:  Port-A als Ausgang konfigurieren (fr LCD-Control)
	//de:     Bit0 = CS = ChipSelect Low-aktiv
	//de:     Bit1 = R/W = Low=Schreiben
	//de:     Bit2 = RS = Low=Control, High=Data
	//de:     Bit3 = E = Schreibflanke H->L
	//de:     Bit4 = Reset = Low-aktiv
	//eng: config port-A as output (for LCD-Control)
	//eng:    bit0 = CS = chip-select low-activ
	//eng:    bit1 = R/W = low=write
	//eng:    bit2 = RS = low=control, high=data
	//eng:    bit3 = E = writing edge H->L
	//eng:    bit4 = Reset = low-aktiv
	PortLcdCtrlDDR=0xff;
	PortLcdCtrl=0b1100011;
}
//----------------------------------------------------------------------
//de:  PE: = Parameter Eingang 
//de:  PA: = Parameter Ausgang
//eng: PE: = parameter input
//eng: PA: = paramerer output 
//----------------------------------------------------------------------
// LCD-Funktionen - Deklaration
//----------------------------------------------------------------------
//de:  initialisiert das LCD
//eng: to initialise the LCD
void lcdInit();
//de:  sendet ein Steuerkommando an das LCD
//de:     PE: data = Steuerkommando
//eng: send a control command to the LCD
//eng:    PE: data = control command
void lcdSendCmd(uint8_t data);
//--------------------
//de:  schreibt ein Byte an die aktuelle Position auf dem LCD (aktuelle Position wird weitergesetzt)
//de:     PE: data = das Byte
//eng: writes a Byte on the actual position at the LCD (actual position is increase by one)
//eng:    PE: data = the Byte
void lcdSendData(uint8_t data);
//--------------------
//de:  liest ein Byte von der aktuellen Position auf dem LCD (aktuelle Position wird beibehalten)
//de:     PA: das glesene Byte
//eng: reads a byte from the actual postion at the LCD (actual position is not changed)
//eng:    PA: the byte that be read
uint8_t lcdReadData();
//--------------------
//de:  Setzt die Position der nchsten Ausgabe, z.B. fr Texte.
//de:  Der y-Wert wird auf eine 8er Grenze abgerundet. (0,8,16,24...56)
//de:     PE: x = x-Koordinate 0...127; y = y-Koordinate 0...63
//eng: Set the position for the next output, for text e. g.
//eng: the y-value is round down to a 8like value (0,8,16,24...56)
//eng:    PE: x = x-coordinate 0...127; y = y-coordinate 0...63  
void lcdSetPos(uint8_t x, uint8_t y);
//--------------------
//de:  rckt die angegebenen Zeile zur aktuellen Zeile auf dem LCD (Rolleffekt)
//de:     PE: line = Nummer der Linie zu der gerollt werden soll
//eng: move to the inputline to the actual line on the LCD (srcolleffect)
//eng:    PE: Line = number of the line to goto
void lcdGotoLine(uint8_t line);
//--------------------
//de:  lscht alle angezeigten Daten auf dem LCD
//eng: clears all data on the LCD
void lcdClear();
//--------------------
//de:  schaltet die Anzeige aus/ein
//de:     PE: on = Ein/Aus 0/1; 0 = Anzeige an; 1 = Anzeige aus (komplett dunkel)
//eng: turn the display on/off
//enge:   PE: on = on/off 0/1; 0 = display on; 1 = display off (complete dark) 
void lcdSendCmdOn(uint8_t on);
//--------------------
//de:  invertiert die Anzeige (dunkel auf hell/ hell auf dunkel)
//de:     PE: on = Aus/Ein 0/1; 0 = Normal (dunkel auf hell); 1 = Invers (hell auf dunkel)
//eng: inverse display (dark at bright backround/ bright at dark backround)
//eng:    PE: on = off/on 0/1; 0 = normal (dark at blight backround); 1 = inverse (blight at dark backround)
void lcdSendCmdReverse(uint8_t jn);
//--------------------
//de:  setzt ein Pixel an Position x,y auf dem LCD
//de:     PE: x = x-Koordinate 0...127; y = y-Koordinate 0...63
//eng: set a Pixel on position x,y at the LCD
//eng:    PE: x = x-coordinate 0...127; y = y-coordinate 0...63
void lcdSetPixel(uint8_t x, uint8_t y);
//----------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////
//----------------------------------------------------------------------
// interne Funktionen
//----------------------------------------------------------------------
//de:  zhlt den Wert von 0 bis x hoch
//de:     PE: x = Hchstwert
//eng: counts a value from 0 to x
//eng:    PE: x = highest index   
void wait(uint16_t x)
{
	//de:  Zhlschleife
	//eng: loop
	while (x>0)
		//de:  x um eins erhhen
		//eng: incraese x by one
		x--;
}
//--------------------
//de: sendet daten an das LCD, wird von lcdSendData() benutzt
//de:    PE: data = zu sendende Daten
//de: send data to the LCD, ist used by lcdSendData()
//de:    PE: data = data to send
void lcdSend(uint8_t data)
{
	//de:  Bit 3 an Port-A lschen (Schreiben auf LCD)
	//eng: bit 3 on port-A clear (write on LCD)
	cbi(PortLcdCtrl,3);
	//de:  Daten auf Port-C
	//eng: data on port-C
	PortLcdData=data;
	//de:  Port-C als Ausgang konfigurieren
	//eng: config port-C as output
	PortLcdDataDDR=0xFF;
	//de:  Bit 0 an Port-A lschen (ChipSelect = An)
	//eng: bit 0 on port-A clear (chipselect = on)
	cbi(PortLcdCtrl,0);
	//de:  Bit 4 an Port-A setzen; E = 1 (Schreiben mit HighPuls an E)
	//eng: set bit 4 on port-A; E = 1 (Write with highpuls on E)
	sbi(PortLcdCtrl,4);
	//de:  Inlineassembler: kurz warten
	//eng: inlineassembler: wait
	__asm volatile ("nop \n nop \n nop \n");
	//de:  Bit 4 an Port-A lschen; E = 0
	//eng: bit 4 on port-A clear; E = 0
	cbi(PortLcdCtrl,4);				
	//de:  Bit 0 an Port-A setzten (ChipSelect = Aus)
	//eng: set bit 0 on port-A (chipselect = off)
	sbi(PortLcdCtrl,0);
}
//----------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////
//----------------------------------------------------------------------
// LCD-Funktionen - Implementation
//----------------------------------------------------------------------
//de:  initialisiert das LCD
//eng: to initialise the LCD
void lcdInit()
{
	//de:  warten 
	//eng: wait
	wait(60000);	
	//de:   Licht aus
	//eng:  light off
	PortLcdCtrl&=~0b00100000;
	//de:  Bit 0 an Port-A setzten (ChipSelect = Aus)
	//eng: set bit 0 on port-A (chipselect = off)
	sbi(PortLcdCtrl,0);
	//de:  Bit 7 an Port-A setzten (parallel on)
	//eng: set bit 7 on port-A (parallel on)
	sbi(PortLcdCtrl,7);
	//de:  Bit 6 an Port-A setzten (6800 Modus)
	//eng: set bit 6 on port-A (6800 mode)
	sbi(PortLcdCtrl,6);

	//de:  Reset am LCD
	//eng: reset the LCD
	PortLcdCtrl&=~0b00000010;
	wait(10000);	
	PortLcdCtrl|=0b00000010;
	wait(10000);	

	//de:  Eine Reihe von Steuerkommandos an das LCD senden (sihe Datenblatt)
	//eng: send different controlcommand to the LCD (datasheet for details)
	
	// internal Reset 
	lcdSendCmd(0xE2);
	// ADC select	
	lcdSendCmd(0xA0);
	// COM output scan direction = revers 	
	lcdSendCmd(0xC8);
	// LCD drive voltage bias ratio 	
	lcdSendCmd(0xA2);
	// no idea 
	lcdSendCmd(0x2F);
	// no idea
	lcdSendCmd(0x26);
	// select booster ratio	
	lcdSendCmd(0xF8);
	// no idea	
	lcdSendCmd(0x00);
	// set output voltage	
	lcdSendCmd(0x81);
	// no idea	
	lcdSendCmd(0x09);
	// read-modify-write	
	lcdSendCmd(0xE0);	
	// display on
	lcdSendCmd(0xAF);	
}
//--------------------
//de:  sendet ein Steuerkommando an das LCD
//de:     PE: data = Steuerkommando
//eng: send a control command to the LCD
//eng:    PE: data = control command
void lcdSendCmd(uint8_t data)
{
	//de:  Bit 2 an Port-A lschen (Datenart = Steuerkommando)
	//eng: bit 2 on port-A clear (datakind = controlcomand)
	cbi(PortLcdCtrl,2);
	//de:  Senden
	//eng: send 
	lcdSend(data);
}
//--------------------
//de:  schreibt ein Byte an die aktuelle Position auf dem LCD (aktuelle Position wird weitergesetzt)
//de:     PE: data = das Byte
//eng: writes a byte on the actual position at the LCD (actual position is increase by one)
//eng:    PE: data = the byte
void lcdSendData(uint8_t data)
{
	
	//de:  Bit 2 an Port-A setzten (Datenart = Daten)
	//eng: set bit 2 on port-A (datakind = data)
	sbi(PortLcdCtrl,2);
	//de:  Senden
	//eng: send 
	lcdSend(data);
}
//--------------------
//de:  liest ein Byte von der aktuellen Position auf dem LCD (aktuelle Position wird beibehalten)
//de:     PA: das Byte
//eng: reads a byte from the actual postion at the LCD (actual position is not changed)
//eng:    PA: the byte
uint8_t lcdReadData()
{
	uint8_t data;

	//de:  Port-C als Eingang konfiguriren (fr lesen)
	//eng: config port-C as input (for read)
	PortLcdDataDDR=0x00;
	//de:  Bit 2 an Port-A setzten (Datenart = Daten)
	//eng: set bit 2 on port-A (datakind = data)
	sbi(PortLcdCtrl,2);
	//de:  Bit 3 an Port-A setzten (lesen)
	//eng: set bit 3 on port-A (read)
	sbi(PortLcdCtrl,3);
	//de:  Bit 0 an Port-A lschen (ChipSelect = An)
	//eng: bit 0 on port-A clear (chipselect = on)
	cbi(PortLcdCtrl,0);

	//de:  ein DUMMY-lesen nach Adress-nderung (MUSS SEIN!)
	//eng: a DUMMY-read after adress-change (MUSTBE!)

	//de:  Bit 4 an Port-A setzten (Flanke vorbereiten)
	//eng: set bit 4 on port-A (create a edge)
	sbi(PortLcdCtrl,4);
	//de:  Inlineassembler: kurz warten
	//eng: inlineassembler: wait
	__asm volatile ("nop \n nop \n nop \n");
	//de:  Bit 4 an Port-A lschen 
	//eng: bit 4 on port-A clear 
	cbi(PortLcdCtrl,4);				

	//de:  Ende DUMMY-lesen
	//eng: end DUMMY-read

	//de:  Bit 4 an Port-A setzten (Flanke vorbereiten)
	//eng: set bit 4 on port-A (create a edge)
	sbi(PortLcdCtrl,4);
	//de:  Inlineassembler: kurz warten
	//eng: inlineassembler: wait
	__asm volatile ("nop \n nop \n nop \n");
	//de:  Daten an PIN-C holen
	//eng: get data on PIN-C
	data=PortLcdDataPIN;
	//de:  Bit 4 an Port-A lschen 
	//eng: bit 4 on port-A clear 
	cbi(PortLcdCtrl,4);				

	//de:  Bit 0 an Port-A setzten (ChipSelect = Aus)
	//eng: set bit 0 on port-A (chipselect = off)
	sbi(PortLcdCtrl,0);
	//de:  Port-C als Ausgang konfigurieren
	//eng: config port-C as output
	PortLcdDataDDR=0xFF;

	return data;
}
//--------------------
//de:  Setzt die Position der nchsten Ausgabe, z.B. fr Texte.
//de:  Der y-Wert wird auf eine 8er Grenze abgerundet. (0,8,16,24...56)
//de:     PE: x = x-Koordinate 0...127; y = y-Koordinate 0...63
//eng: Set the position for the next output, for text e. g.
//eng: the y-value is round down to a 8like value (0,8,16,24...56)
//eng:    PE: x = x-coordinate 0...127; y = y-coordinate 0...63  
void lcdSetPos(uint8_t x, uint8_t y)
{
	//de:  x-Koordinate
	//eng: x-coordinate
	lcdSendCmd(0b00010000+(x>>4));
	lcdSendCmd(0b00000000+(x&0x0f));
	//de:  y-Koordinate
	//eng: y-coordinate
	lcdSendCmd(0b10110000+(y>>3));
}
//--------------------
//de:  rckt die angegebenen Zeile zur aktuellen Zeile auf dem LCD (Rolleffekt)
//de:     PE: line = Nummer der Linie zu der gerollt werden soll
//eng: move to the inputline to the actual line on the LCD (srcolleffect)
//eng:    PE: Line = number of the line to goto
void lcdGotoLine(uint8_t line)
{
	//de:  Zeile initialisieren
	//eng: initial display line
	lcdSendCmd(0b01000000+(line&0b00111111));
}
//--------------------
//de:  lscht alle angezeigten Daten auf dem LCD
//eng: clears all data on the LCD
void lcdClear()
{
	//de:  die Anzeige besteht 128 * 8 Bytes, welche senkrecht angeordnet sind
	//de:  in jedem Byte liegt Bit0 oben und Bit7 unten
	//eng: the Display consists of 128 * 8 Bytes, which are placed vertical
	//eng: in each byte are bit0 up and bit7 down 
	
	//de:  y-Schleife (senkrecht); 8 mal, da 64 Pixel aus 8 Bytes je 8 Bits bestehen
	//eng: y-loop (vertical); 8 times, 64 pixels are 8 bytes * 8 bits   
	for(uint8_t y=0;y<64;y+=8)
	{
	    //de:  gehe zu Zeilenanfang
	    //eng: goto linebegin
		lcdSetPos(0,y);
		//de:  x-Schleife (waagerecht); 128 Pixel sind 128 
		for(uint16_t x=0;x<128;x++)
			//de:  Byte auf 0 setzen
			//eng: set byte to 0
			lcdSendData(0);
	}
	//de:  aktuelle Position auf linksoben setzen (0,0)
	//de:  set actual position to leftupper corner (0,0)
	lcdSetPos(0,0);
}
//--------------------
//de:  invertiert die Anzeige (dunkel auf hell/ hell auf dunkel)
//de:     PE: on = Aus/Ein 0/1; 0 = Normal (dunkel auf hell); 1 = Invers (hell auf dunkel)
//eng: inverse display (dark at bright backround/ bright at dark backround)
//eng:    PE: on = off/on 0/1; 0 = normal (dark at blight backround); 1 = inverse (blight at dark backround)
void lcdSendCmdReverse(uint8_t jn)
{
	//de:  Anzeige normal/revers
	//eng: display normal/reverse
	lcdSendCmd(0b10100110+jn);
}
//--------------------
//de:  schaltet die Anzeige aus/ein
//de:     PE: on = Ein/Aus 0/1; 0 = Anzeige an; 1 = Anzeige aus (komplett dunkel)
//eng: turn the display on/off
//eng:    PE: on = on/off 0/1; 0 = display on; 1 = display off (complete dark) 
void lcdSendCmdOn(uint8_t on)
{
	//de:  Alle Pixel an/aus
	//eng: all pixels on /off 
	lcdSendCmd(0b10100100+on);
}
//--------------------
//de:  setzt ein Pixel an Position x,y auf dem LCD
//de:     PE: x = x-Koordinate 0...127; y = y-Koordinate 0...63
//eng: set a Pixel on position x,y at the LCD
//eng:    PE: x = x-coordinate 0...127; y = y-coordinate 0...63
void lcdSetPixel(uint8_t x, uint8_t y)
{
	//de:  gehe zur Position
	//eng: goto position
	lcdSetPos(x,y);
	uint8_t data;
	//de:  Bitmuster fr Position  des Bits (Pixel) im Byte
	//eng: bitmask for the bit (pixel) in the byte
	data=1<<(y&0b00000111);
	//de:  Byte auslesen, Bit setzten, Byte zurckschreiben
	//eng: read the byte, set the bit, write the byte
	lcdSendData( lcdReadData()|data );
}
//--------------------

// Die Bytes liegen senkrecht von links nach rechts, dann zeilenweisen nach unten
// in jedem Byte liegt Bit0 oben und Bit7 unten
uint8_t zeichensatz[]={
	0b11111110,0b00010000,0b00010000,0b00010000,0b11111110,0,	/* H */
	0b01100000,0b10010000,0b10010000,0b10010000,0b11110000,0,	/* a */
	0b00000000,0b00000000,0b11111110,0b00000000,0b00000000,0,	/* l */
	0b00000000,0b00000000,0b11111110,0b00000000,0b00000000,0,   /* l */
	0b01100000,0b10010000,0b10010000,0b10010000,0b01100000,0,   /* o */
	0
};
//de:  Hauptfunktion
//eng: mainfunction
main()
{
	// Init
	init(); 	
	PortLed=0x00;
	char x;
	PortLed=1;

	lcdInit();
	lcdClear();
	PortLcdCtrl|=0b00100000;	// Licht an
//--------------------
    //de:  LED eine weiter
    //eng: next LED 
	PortLed *=2;
	//de:  1sek warten
	//eng: wait 1sec
	waitMs(1000);	
//--------------------
	//de:  ein Dreieck an Position 0,0 zeichnen
	//eng: draw a triangle on position 0,0	
	
	
	//de:  setzen der Position
	//eng: set the position
	lcdSetPos(0,0);
	//de:  senden der Daten
	//eng: send  the data	
	lcdSendData(0x80);	
	lcdSendData(0xC0);	
	lcdSendData(0xE0);	
	lcdSendData(0xF0);	
	lcdSendData(0xF8);	
	lcdSendData(0xFC);	
	lcdSendData(0xFE);	
	lcdSendData(0xFF);
//--------------------
    //de:  LED eine weiter
    //eng: next LED 
	PortLed *=2;
	//de:  1sek warten
	//eng: wait 1sec
	waitMs(1000);	
//--------------------
	//de:  von 0 bis 127 zhlen und in der Zeile 3 die Werte als Daten ausgeben
	//eng: counts from 0 to 127 und output the value in a line 3 as data  
	
	//de:  setzen der Position
	//eng: set the position
	lcdSetPos(0,16);
	//de:  Zhlschleife
	//eng: loop	
	for(uint8_t c=1;c<=128;c++)
	{
		//de:  Senden des Werts
		//eng: send the value
		lcdSendData(x++);
		//de:  warten
		//eng: wait	
		wait(10000);	
	}
//--------------------
    //de:  LED eine weiter
    //eng: next LED 
	PortLed *=2;
	//de:  1sek warten
	//eng: wait 1sec
	waitMs(1000);	
//--------------------
	//de:  schreibt "Hallo" an Position 21,32
	//eng: write on position 21,32 "Hallo"	
	
	//de:  setzen der Position
	//eng: set the position
	lcdSetPos(21,32);
	uint8_t* ptxt=zeichensatz;
	//de:  Daten aus zeichsatz ausgeben
	//eng: output data from zeichenstz
	for(uint8_t n=0;n<5;n++)
		for(uint8_t s=0;s<6;s++)
		{
			lcdSendData(ptxt[0]);
			ptxt++;
		}
//--------------------
    //de:  LED eine weiter
    //eng: next LED 
	PortLed *=2;
	//de:  1sek warten
	//eng: wait 1sec
	waitMs(1000);	
//--------------------	
	//de:  ein "!" hinter das Hallo schreiben
	//eng: writes a "!" behind the Hallo
	  
	lcdSendData(0xBE);
//--------------------
    //de:  LED eine weiter
    //eng: next LED 
	PortLed *=2;
	//de:  1sek warten
	//eng: wait 1sec
	waitMs(1000);	
//--------------------	
	//de:  eine Line horizontal zeichnen
	//eng: draw a horizontal line
	
	uint8_t y=48;
	for(uint8_t x=0;x<30;x++)
		lcdSetPixel(x,y);
		
	//de:  eine Line schrg zeichnen
	//eng: draw a bevel line
	
	y=42;
	for(uint8_t x=10;x<30;x++)
	{
		lcdSetPixel(x,y);
		y++;
	}
//--------------------
    //de:  LED eine weiter
    //eng: next LED 
	PortLed *=2;
	//de:  1sek warten
	//eng: wait 1sec
	waitMs(1000);	
//--------------------		
	// Mainloop
	
	//de:  die Anzeige rollen lassen
	//eng: scroll the Display
	y=0;
	lcdSetPos(0,0);
	while (true)  	// Mainloop-Begin
	{
		lcdGotoLine(y);
		waitMs(50);
		y++;
	}
}
//---------------------------------------------------------------------------

